@@ -1,14 +1,3 @@
-//! A caching middleware that follows HTTP caching rules, thanks to
-//! [`http-cache-semantics`](https://github.com/kornelski/rusty-http-cache-semantics).
-//! By default, it uses [`cacache`](https://github.com/zkat/cacache-rs) as the backend cache manager.
-//!
-//! ## Features
-//!
-//! The following features are available. By default `manager-cacache` is enabled.
-//!
-//! - `manager-cacache` (default): use [cacache](https://github.com/zkat/cacache-rs),
-//! a high-performance disk cache, for the manager backend.
-//! - `with-http-types` (disabled): enable [http-types](https://github.com/http-rs/http-types) type conversion support
 #![forbid(unsafe_code, future_incompatible)]
 #![deny(
     missing_docs,
@@ -18,10 +7,24 @@
     unused_qualifications,
     unused_import_braces,
     unused_extern_crates,
-    rustdoc::missing_doc_code_examples,
     trivial_casts,
     trivial_numeric_casts
 )]
+#![cfg_attr(docsrs, feature(doc_cfg))]
+//! A caching middleware that follows HTTP caching rules, thanks to
+//! [`http-cache-semantics`](https://github.com/kornelski/rusty-http-cache-semantics).
+//! By default, it uses [`cacache`](https://github.com/zkat/cacache-rs) as the backend cache manager.
+//!
+//! ## Features
+//!
+//! The following features are available. By default `manager-cacache` is enabled.
+//!
+//! - `manager-cacache` (default): enable [cacache](https://github.com/zkat/cacache-rs),
+//! a high-performance disk cache, backend manager.
+//! - `manager-moka` (disabled): enable [moka](https://github.com/moka-rs/moka),
+//! a high-performance in-memory cache, backend manager.
+//! - `with-http-types` (disabled): enable [http-types](https://github.com/http-rs/http-types)
+//! type conversion support
 mod error;
 mod managers;
 
@@ -39,8 +42,16 @@ pub use error::{CacheError, Result};
 #[cfg(feature = "manager-cacache")]
 pub use managers::cacache::CACacheManager;
 
+#[cfg(feature = "manager-moka")]
+pub use managers::moka::MokaManager;
+
+// Exposing the moka cache for convenience, renaming to avoid naming conflicts
+#[cfg(feature = "manager-moka")]
+#[cfg_attr(docsrs, doc(cfg(feature = "manager-moka")))]
+pub use moka::future::{Cache as MokaCache, CacheBuilder as MokaCacheBuilder};
+
 /// Represents an HTTP version
-#[derive(Debug, Copy, Clone, Deserialize, Serialize)]
+#[derive(Debug, Copy, Clone, PartialEq, Deserialize, Serialize)]
 #[non_exhaustive]
 pub enum HttpVersion {
     /// HTTP Version 0.9
@@ -189,7 +200,7 @@ pub trait Middleware {
     /// Attempts to construct `http::request::Parts` from the request
     fn parts(&self) -> Result<request::Parts>;
     /// Attempts to determine the requested url
-    fn url(&self) -> Result<&Url>;
+    fn url(&self) -> Result<Url>;
     /// Attempts to determine the request method
     fn method(&self) -> Result<String>;
     /// Attempts to fetch an upstream resource and return an [`HttpResponse`]
@@ -313,7 +324,7 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
             return middleware.remote_fetch().await;
         }
         if let Some(store) =
-            self.manager.get(&middleware.method()?, middleware.url()?).await?
+            self.manager.get(&middleware.method()?, &middleware.url()?).await?
         {
             let (mut res, policy) = store;
             let res_url = res.url.clone();
@@ -356,13 +367,13 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
             match self.mode {
                 CacheMode::OnlyIfCached => {
                     // ENOTCACHED
-                    return Ok(HttpResponse {
+                    Ok(HttpResponse {
                         body: b"GatewayTimeout".to_vec(),
                         headers: Default::default(),
                         status: 504,
-                        url: middleware.url()?.clone(),
+                        url: middleware.url()?,
                         version: HttpVersion::Http11,
-                    });
+                    })
                 }
                 _ => self.remote_fetch(&mut middleware).await,
             }
@@ -386,11 +397,11 @@ impl<T: CacheManager + Send + Sync + 'static> HttpCache<T> {
         if is_cacheable {
             Ok(self
                 .manager
-                .put(&middleware.method()?, middleware.url()?, res, policy)
+                .put(&middleware.method()?, &middleware.url()?, res, policy)
                 .await?)
         } else if !middleware.is_method_get_head() {
             self.manager
-                .delete(&middleware.method()?, middleware.url()?)
+                .delete(&middleware.method()?, &middleware.url()?)
                 .await?;
             Ok(res)
         } else {
